package InstallUtils;

use strict;
use vars qw($VERSION @ISA @EXPORT);
use Term::ReadLine;
use FileHandle;

@ISA = qw(Exporter);

@EXPORT = qw(
RequestInput
Log
);

our $logFile = "/tmp/cuda-installer-$$";
my $logStarted = 0;
local *LOG;

$VERSION = '1.0';

my $gSilent = 0;
my $gVerbose = 0;
my $gTerm = new Term::ReadLine 'CUDA Installer';

$gTerm->ornaments(0);

autoflush STDOUT 1;

#
# This subroutine takes a prompt and gets input from the user regarding the prompt.
# If options are included, the user's input must match one of the users. Shortcuts,
# if not supressed, can also be used for options that have unique first characters.
# Lastly, a default value can be specified if the user enters the empty string.
#
# Parameters:
#     $PROMPT, @OPTIONS, $SUPPRESS_SHORTCUTS, $DEFAULT
#
sub RequestInput
{
    my %params = @_;

    my $prompt = $params{PROMPT};
    my @options = @{$params{OPTIONS}} if (defined $params{OPTIONS});
    my $suppress_shortcuts = $params{SUPPRESS_SHORTCUTS};
    my $default = $params{DEFAULT};

    return "" if (!defined $prompt || $prompt =~ m/^\s+$/);

    #
    # Need to see if any options share a first character, split them into arrays
    # based on this property
    #
    my %seen_chars;
    my %full_string_options_orderinging;
    my @full_string_options;
    my %shortcut_options_ordering;
    my @shortcut_options;

    if (!$suppress_shortcuts)
    {
        foreach my $option (@options)
        {
            my $char = substr($option,0,1);

            if (exists $seen_chars{$char})
            {
                $seen_chars{$char} = $seen_chars{$char} + 1;
            }
            else
            {
                $seen_chars{$char} = 1;
            }
        }
    }

    #
    # Maintain a counter for each variable, as we want to preserve the order they appear
    # as the order they were requested in the parameter
    #
    my $count = 0;
    foreach my $option (@options)
    {
        my $char = substr($option,0,1);

        if ($suppress_shortcuts || $seen_chars{$char} > 1)
        {
            push (@full_string_options, $option);
            $full_string_options_orderinging{$count} = $option;
        }
        else
        {
            push (@shortcut_options, $option);
            $shortcut_options_ordering{$count} = $option;
        }

        $count++;
    }

    $prompt .= " (" if (@options);

    for (my $iter = 0; $iter < $count; $iter++)
    {
        if (exists $shortcut_options_ordering{$iter})
        {
            my $cur_option = $shortcut_options_ordering{$iter};
            my $first_char = substr($cur_option,0,1);
            my $rest_of_option = substr($cur_option,1,length($cur_option) - 1);
            $prompt .= "($first_char)$rest_of_option";
        }
        else
        {
            my $cur_option = $full_string_options_orderinging{$iter};
            $prompt .= "$cur_option";
        }

        $prompt .= "/" if ($iter + 1 != $count);
    }

    $prompt .= ")" if (@options);

    if ($default)
    {
        $prompt .= " [ default is $default ]: ";
    }
    else
    {
        $prompt .= ": ";
    }

    while (defined (my $input = $gTerm->readline($prompt)))
    {
        #
        # Trim whitespace off both ends of input
        #
        $input =~ s/^\s+//;
        $input =~ s/\s+$//;

        if ($input eq "" && $default)
        {
            return $default;
        }

        if (!@options)
        {
            $input =~ s|/+|/|g;
            return $input;
        }

        foreach my $option (@options)
        {
            my $found = 0;

            foreach my $shortcut (@shortcut_options)
            {
                if ($option eq $shortcut)
                {
                    $found = 1;
                    last;
                }
            }

            if ($found)
            {
                my $first_char = substr($option,0,1);

                if ($input =~ m/^$first_char$/i || $input =~ m/^$option$/i)
                {
                    return $option;
                }
            }
            else
            {
                if ($input =~ m/^$option$/i)
                {
                    return $option;
                }
            }
        }
    }
}

#
# Set verbose flag. Default is 0.
#
sub SetVerbose
{
    my $verbose = shift;

    $gVerbose = $verbose if (defined $verbose);
}

#
# Set silent flag. Default is 0.
#
sub SetSilent
{
    my $silent = shift;

    $gSilent = $silent if (defined $silent);
}

#
# Usage:
# Log("text", <IsImportant>);
#
# If <IsImportant> is not passed, assume not important.
# <IsImportant> = 1 tells the engine to print to stdout if silent.
#
sub Log
{
    if (!$logStarted)
    {
        open (LOG, ">", "$logFile") or die "Unable to open $logFile to write: $!";
        $logStarted = 1;

        print "Logging to $logFile\n" if (!$gSilent);
    }

    my $text = shift;
    my $important = 0;
    my $explicit = 0;
    
    if (scalar(@_) >= 1)
    {
        $important = shift;
        $explicit = 1;
    }

    print LOG "$text\n";

    #
    # Print if:
    #  we're verbose
    #  we're not silent and not told anything about importance of text
    #  we're told the text is important
    #
    if ($gVerbose || (!$gSilent && !$explicit) || ($explicit && $important))
    {
        print "$text\n";
    }
}

END
{
    close LOG;
}

1;

__END__

=head1 NAME

InstallUtils - CUDA Installer Utilities

=head1 SYNOPSIS

    use InstallUtils;

=head1 DESCRIPTION

Provides various installation utilities:

    request_input - Subroutine for getting input from the user
    log           - Subroutine for logging based on a state

=head1 DETAILS

=head2 request_input

    @Options = ("yes", "no", "maybe");
    $default = "no";
    &request_input(
                   PROMPT => "Prompt text for the user",
                   OPTIONS => \@Options,
                   SUPPRESS_SHORTCUTS => (0|1),
                   DEFAULT => $default,
                  );

    Parameters can be passed in any order, and only prompt is required.
